"""

"""
import logging

from ..util.lockdown import LockdownClient


class DiagnosticsRelayService:
    SERVICE_NAME = "com.apple.mobile.diagnostics_relay"

    def __init__(self, lockdown=None, udid=None, network=None, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown if lockdown else LockdownClient(udid=udid, network=network)
        self.conn = self.lockdown.start_service("com.apple.mobile.diagnostics_relay")

    def get_battery(self, name='IOPMPowerSource'):
        ret = self.conn.plist_request({
            'Request': 'IORegistry',
            'EntryClass': name,
        })
        return ret['Diagnostics']["IORegistry"]

    def reboot(self) -> str:
        """ reboot device """
        ret = self.conn.plist_request({
            "Request": "Restart",
        })
        return ret['Status']

    def shutdown(self):
        ret = self.conn.plist_request({
            "Request": "Shutdown",
        })
        return ret['Status']



# region operations
AFC_OP_STATUS          = 0x00000001
AFC_OP_DATA            = 0x00000002     # Data */
AFC_OP_READ_DIR        = 0x00000003     # ReadDir */
AFC_OP_READ_FILE       = 0x00000004     # ReadFile */
AFC_OP_WRITE_FILE      = 0x00000005     # WriteFile */
AFC_OP_WRITE_PART      = 0x00000006     # WritePart */
AFC_OP_TRUNCATE        = 0x00000007     # TruncateFile */
AFC_OP_REMOVE_PATH     = 0x00000008     # RemovePath */
AFC_OP_MAKE_DIR        = 0x00000009     # MakeDir */
AFC_OP_GET_FILE_INFO   = 0x0000000a     # GetFileInfo */
AFC_OP_GET_DEVINFO     = 0x0000000b     # GetDeviceInfo */
AFC_OP_WRITE_FILE_ATOM = 0x0000000c     # WriteFileAtomic (tmp file+rename) */
AFC_OP_FILE_OPEN       = 0x0000000d     # FileRefOpen */
AFC_OP_FILE_OPEN_RES   = 0x0000000e     # FileRefOpenResult */
AFC_OP_READ            = 0x0000000f     # FileRefRead */
AFC_OP_WRITE           = 0x00000010     # FileRefWrite */
AFC_OP_FILE_SEEK       = 0x00000011     # FileRefSeek */
AFC_OP_FILE_TELL       = 0x00000012     # FileRefTell */
AFC_OP_FILE_TELL_RES   = 0x00000013     # FileRefTellResult */
AFC_OP_FILE_CLOSE      = 0x00000014     # FileRefClose */
AFC_OP_FILE_SET_SIZE   = 0x00000015     # FileRefSetFileSize (ftruncate) */
AFC_OP_GET_CON_INFO    = 0x00000016     # GetConnectionInfo */
AFC_OP_SET_CON_OPTIONS = 0x00000017     # SetConnectionOptions */
AFC_OP_RENAME_PATH     = 0x00000018     # RenamePath */
AFC_OP_SET_FS_BS       = 0x00000019     # SetFSBlockSize (0x800000) */
AFC_OP_SET_SOCKET_BS   = 0x0000001A     # SetSocketBlockSize (0x800000) */
AFC_OP_FILE_LOCK       = 0x0000001B     # FileRefLock */
AFC_OP_MAKE_LINK       = 0x0000001C     # MakeLink */
AFC_OP_GET_FILE_HASH   = 0x0000001D     # GetFileHash */
AFC_OP_SET_FILE_TIME   = 0x0000001E     # set st_mtime */
AFC_OP_GET_FILE_HASH_RANGE       = 0x0000001F   # GetFileHashWithRange */
# iOS 6+ */
AFC_OP_FILE_SET_IMMUTABLE_HINT   = 0x00000020	# FileRefSetImmutableHint */
AFC_OP_GET_SIZE_OF_PATH_CONTENTS = 0x00000021	# GetSizeOfPathContents */
AFC_OP_REMOVE_PATH_AND_CONTENTS  = 0x00000022	# RemovePathAndContents */
AFC_OP_DIR_OPEN                  = 0x00000023	# DirectoryEnumeratorRefOpen */
AFC_OP_DIR_OPEN_RESULT           = 0x00000024	# DirectoryEnumeratorRefOpenResult */
AFC_OP_DIR_READ                  = 0x00000025	# DirectoryEnumeratorRefRead */
AFC_OP_DIR_CLOSE                 = 0x00000026	# DirectoryEnumeratorRefClose */
# iOS 7+ */
AFC_OP_FILE_READ_OFFSET          = 0x00000027	# FileRefReadWithOffset */
AFC_OP_FILE_WRITE_OFFSET         = 0x00000028	# FileRefWriteWithOffset */
# endregion

# region error codes
AFC_E_SUCCESS                = 0
AFC_E_UNKNOWN_ERROR          = 1
AFC_E_OP_HEADER_INVALID      = 2
AFC_E_NO_RESOURCES           = 3
AFC_E_READ_ERROR             = 4
AFC_E_WRITE_ERROR            = 5
AFC_E_UNKNOWN_PACKET_TYPE    = 6
AFC_E_INVALID_ARG            = 7
AFC_E_OBJECT_NOT_FOUND       = 8
AFC_E_OBJECT_IS_DIR          = 9
AFC_E_PERM_DENIED            = 10
AFC_E_SERVICE_NOT_CONNECTED  = 11
AFC_E_OP_TIMEOUT             = 12
AFC_E_TOO_MUCH_DATA          = 13
AFC_E_END_OF_DATA            = 14
AFC_E_OP_NOT_SUPPORTED       = 15
AFC_E_OBJECT_EXISTS          = 16
AFC_E_OBJECT_BUSY            = 17
AFC_E_NO_SPACE_LEFT          = 18
AFC_E_OP_WOULD_BLOCK         = 19
AFC_E_IO_ERROR               = 20
AFC_E_OP_INTERRUPTED         = 21
AFC_E_OP_IN_PROGRESS         = 22
AFC_E_INTERNAL_ERROR         = 23

AFC_E_MUX_ERROR              =30
AFC_E_NO_MEM                 =31
AFC_E_NOT_ENOUGH_DATA        =32
AFC_E_DIR_NOT_EMPTY          =33
# endregion

AFC_FOPEN_RDONLY   = 0x00000001 #/**< r   O_RDONLY */
AFC_FOPEN_RW       = 0x00000002 #/**< r+  O_RDWR   | O_CREAT */
AFC_FOPEN_WRONLY   = 0x00000003 #/**< w   O_WRONLY | O_CREAT  | O_TRUNC */
AFC_FOPEN_WR       = 0x00000004 #/**< w+  O_RDWR   | O_CREAT  | O_TRUNC */
AFC_FOPEN_APPEND   = 0x00000005 #/**< a   O_WRONLY | O_APPEND | O_CREAT */
AFC_FOPEN_RDAPPEND = 0x00000006 #/**< a+  O_RDWR   | O_APPEND | O_CREAT */

AFC_HARDLINK = 1
AFC_SYMLINK = 2

AFC_LOCK_SH = 1 | 4  #/**< shared lock */
AFC_LOCK_EX = 2 | 4  #/**< exclusive lock */
AFC_LOCK_UN = 8 | 4  #/**< unlock */

AFCMAGIC = b'CFA6LPAA'

AFC_ERRORS = {k: v for k, v in locals().items() if k.startswith('AFC_E_')}
AFC_FILE_MODES = {k: v for k, v in locals().items() if k.startswith('AFC_FOPEN_')}
AFC_OPERATIONS = {k: v for k, v in locals().items() if k.startswith('AFC_OP_')}

AFC_ERROR_NAMES = {v: k for k, v in AFC_ERRORS.items()}
AFC_FILE_MODE_NAMES = {v: k for k, v in AFC_FILE_MODES.items()}
AFC_OPERATION_NAMES = {v: k for k, v in AFC_OPERATIONS.items()}



"""
@Date    : 2021-02-24
@Author  : liyachao
"""
import time

from ios_device import py_ios_device
from ios_device.py_ios_device import PyiOSDevice

xcuitest_bundle_id = "cn.xx.autotest.xctrunner"
app_bundle_id = "com.tencent.xin"


def callback(res):
    print(res)


def test_get_process():
    process = py_ios_device.get_processes()
    print(process)


def test_get_process_obj():
    channel = PyiOSDevice()
    process = channel.get_processes()
    print(process)
    channel.stop()


def test_get_channel():
    channels = py_ios_device.get_channel()
    print(channels)


def test_get_channel_obj():
    channel = PyiOSDevice()
    channels = channel.get_channel()
    print(channels)
    channel.stop()


def test_start_get_gpu():
    channel = py_ios_device.start_get_gpu(callback=callback)
    time.sleep(5)
    py_ios_device.stop_get_gpu(channel)
    channel.stop()


def test_start_get_gpu_obj():
    channel = PyiOSDevice()
    channel.start_get_gpu(callback=callback)
    time.sleep(5)
    channel.stop_get_gpu()
    channel.stop()


def test_get_device():
    device = py_ios_device.get_device()
    app_list = device.get_apps()
    print(app_list)
    for index in app_list:
        print(index.get("CFBundleDisplayName"))
        print(index.get("CFBundleIdentifier"))


def test_get_device_obj():
    channel = PyiOSDevice()
    device = channel.get_device()
    app_list = device.get_apps()
    print(app_list)
    channel.stop()


def test_launch_app():
    py_ios_device.launch_app(app_bundle_id)


def test_launch_app_obj():
    channel = PyiOSDevice()
    channel.launch_app(app_bundle_id)
    channel.stop()


def test_get_applications():
    app_list = py_ios_device.get_applications()
    print(app_list)


def test_get_applications_obj():
    channel = PyiOSDevice()
    app_list = channel.get_applications()
    print(app_list)
    channel.stop()


def test_start_xcuitest():
    channel = py_ios_device.start_xcuitest(bundle_id=xcuitest_bundle_id, callback=callback)
    time.sleep(5)
    py_ios_device.stop_xcuitest(channel)
    channel.stop()


def test_start_xcuitest_obj():
    channel = PyiOSDevice()
    channel.start_xcuitest(bundle_id=xcuitest_bundle_id, callback=callback)
    time.sleep(5)
    channel.stop_xcuitest()
    channel.stop()


def test_start_get_fps():
    channel = py_ios_device.start_get_fps(callback=callback)
    time.sleep(5)
    py_ios_device.stop_get_fps(channel)
    channel.stop()


def test_start_get_fps_obj():
    channel = PyiOSDevice()
    channel.start_get_fps(callback=callback)
    time.sleep(5)
    channel.stop_get_fps()
    channel.stop()


def test_start_get_graphics_fps():
    channel = py_ios_device.start_get_graphics_fps(callback=callback)
    time.sleep(5)
    py_ios_device.stop_get_graphics_fps(channel)
    channel.stop()


def test_start_get_graphics_fps_obj():
    channel = PyiOSDevice()
    channel.start_get_graphics_fps(callback=callback)
    time.sleep(5)
    channel.stop_get_graphics_fps()
    channel.stop()


def test_start_get_mobile_notifications():
    channel = py_ios_device.start_get_mobile_notifications(callback=callback)
    time.sleep(5)
    py_ios_device.stop_get_mobile_notifications(channel)
    channel.stop()


def test_start_get_mobile_notifications_obj():
    channel = PyiOSDevice()
    channel.start_get_mobile_notifications(callback=callback)
    time.sleep(5)
    channel.stop_get_mobile_notifications()
    channel.stop()


def test_get_netstat():
    netstat = py_ios_device.get_netstat(216)
    print(netstat)


def test_get_netstat_obj():
    channel = PyiOSDevice()
    netstat = channel.get_netstat(216)
    print(netstat)
    channel.stop()


if __name__ == "__main__":
    # test_get_process()
    # test_get_process_obj()
    # test_get_channel()
    # test_get_channel_obj()
    # test_start_get_gpu()
    # test_start_get_gpu_obj()
    # test_get_device()
    # test_get_device_obj()
    # test_launch_app()
    # test_launch_app_obj()
    # test_get_applications()
    # test_get_applications_obj()
    # test_start_xcuitest()
    # test_start_xcuitest_obj()
    # test_start_get_fps()
    # test_start_get_fps_obj()
    # test_start_get_graphics_fps()
    # test_start_get_graphics_fps_obj()
    # test_start_get_mobile_notifications()
    # test_start_get_mobile_notifications_obj()
    # test_get_netstat()
    test_get_netstat_obj()



import logging
import os
import sys

import coloredlogs

gettrace = getattr(sys, 'gettrace', None)


def hexdump(d):
    for i in range(0, len(d), 16):
        data = d[i:i + 16]
        print("%08X | %s | %s" % (i, hex(data).ljust(47), ascii(data)))


def read_file(filename):
    f = open(filename, "rb")
    data = f.read()
    f.close()
    return data


coloredlogs.DEFAULT_FIELD_STYLES = {'asctime': {'color': 'green'}, 'hostname': {'color': 'magenta'},
                                    'levelname': {'color': 'green', 'bold': True},
                                    'name': {'color': 'blue'}, 'programname': {'color': 'cyan'},
                                    'threadName': {'color': 'yellow'}}


class Log:
    __instances = {}
    def __init__(self,level = None):
        self.level = level

    @classmethod
    def getLogger(cls, name=os.path.abspath(__name__)):
        if name not in cls.__instances:
            logger = logging.getLogger(name)
            fmt = '%(asctime)s [%(levelname)s] [%(name)s] %(filename)s[line:%(lineno)d] %(message)s'
            formater = logging.Formatter(fmt)
            ch = logging.StreamHandler()
            ch.setLevel(Log.__getLogLevel())
            ch.setFormatter(formater)
            logger.addHandler(ch)
            coloredlogs.install(fmt=fmt, level=Log.__getLogLevel(), logger=logger)
            logger.setLevel(Log.__getLogLevel())
            cls.__instances[name] = logger
        return cls.__instances[name]

    @staticmethod  # 设置日志等级
    def __getLogLevel():
        if os.getenv("DEBUG") in ("1", "on", "true"):
            return logging.DEBUG
        if os.getenv("ERROR") in ("1", "on", "true"):
            return logging.ERROR
        if gettrace():
            return logging.DEBUG
        else:
            return logging.INFO


PROGRAM_NAME = "py_ios_device"



"""
获取 gup 数据，返回时间序列还没琢磨出来怎么解析

startCollectingCounters 之后会返回参数详解之类的， 一些解析方式，但是看起来依然很费劲
"""
import os
import sys

from ios_device.util.dtx_msg import RawInt64sl

sys.path.append(os.getcwd())

from ios_device.servers.Instrument import InstrumentServer

import time
from ios_device.util import logging

log = logging.getLogger(__name__)


def gup(rpc):

    def dropped_message(res):
        print( res.selector,res.auxiliaries)


    # print(rpc.lockdown.device_id)
    rpc.register_undefined_callback(dropped_message)
    print(rpc.call('com.apple.instruments.server.services.deviceinfo','machTimeInfo').selector)
    print(rpc.call('com.apple.instruments.server.services.gpu','requestDeviceGPUInfo').selector)
    rpc.call("com.apple.instruments.server.services.gpu", "configureCounters:counterProfile:interval:windowLimit:tracingPID:",RawInt64sl(0, 3, 0, -1), 4294967295)
    print(rpc.call('com.apple.instruments.server.services.gpu', 'startCollectingCounters').selector)
    # rpc.call("com.apple.instruments.server.services.gpu", "")
    time.sleep(1000)

    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    gup(rpc)
    rpc.stop()



## run xcuitest
## https://github.com/alibaba/taobao-iphone-device/blob/main/tidevice/_device.py#L921
import logging
import os
import threading
import time
from distutils.version import LooseVersion

from ios_device.servers.dvt import DTXEnum
from ios_device.servers.Installation import InstallationProxyService
from ios_device.servers.Instrument import InstrumentServer
from ios_device.servers.house_arrest import HouseArrestService
from ios_device.servers.testmanagerd import TestManagerdLockdown
from ios_device.util.bpylist2 import archive, XCTestConfiguration, NSURL, NSUUID

from ios_device.util.dtx_msg import RawObj
from ios_device.util.lockdown import LockdownClient

logging.basicConfig(level=logging.INFO)


class RunXCUITest(threading.Thread):
    def __init__(self, bundle_id, udid=None):
        super().__init__()
        self.udid = udid
        self.bundle_id = bundle_id
        self.quit_event = threading.Event()

    def close(self):
        self.quit_event.set()

    def run(self) -> None:
        def _callback(res):
            logging.info(f" {res.selector} : {res.auxiliaries}")

        self.lockdown = LockdownClient(udid=self.udid, network=False)
        installation = InstallationProxyService(lockdown=self.lockdown)
        app_info = installation.find_bundle_id(self.bundle_id)
        if not app_info:
            raise Exception("No app matches", self.bundle_id)
        logging.info("BundleID: %s", self.bundle_id)
        logging.info("DeviceIdentifier: %s", self.lockdown.device_info.get('UniqueDeviceID'))
        sign_identity = app_info.get("SignerIdentity", "")
        logging.info("SignIdentity: %r", sign_identity)
        XCODE_VERSION = 29
        session_identifier = NSUUID(bytes=os.urandom(16), version=4)
        ManagerdLockdown1 = TestManagerdLockdown(self.lockdown).init()

        ManagerdLockdown1.make_channel("dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface")
        if self.lockdown.ios_version > LooseVersion('11.0'):
            result = ManagerdLockdown1.call(
                "dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface",
                "_IDE_initiateControlSessionWithProtocolVersion:", RawObj(XCODE_VERSION)).selector
            logging.info("result: %s", result)
        ManagerdLockdown1.register_selector_callback(DTXEnum.FINISHED, lambda _: self.quit_event.set())
        ManagerdLockdown1.register_undefined_callback(_callback)

        ManagerdLockdown2 = TestManagerdLockdown(self.lockdown).init()
        ManagerdLockdown2.make_channel("dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface")
        ManagerdLockdown2.register_selector_callback(DTXEnum.FINISHED, lambda _: self.quit_event.set())
        ManagerdLockdown2.register_undefined_callback(_callback)

        _start_flag = threading.Event()

        def _start_executing(res=None):
            if _start_flag.is_set():
                return
            _start_flag.set()

            logging.info("Start execute test plan with IDE version: %d",
                         XCODE_VERSION)
            ManagerdLockdown2._call(False, 0xFFFFFFFF, '_IDE_startExecutingTestPlanWithProtocolVersion:',
                                    RawObj(XCODE_VERSION))

        def _show_log_message(res):
            logging.info(f"{res.selector} : {res.auxiliaries}")
            if 'Received test runner ready reply with error: (null' in ''.join(res.auxiliaries):
                logging.info("Test runner ready detected")
                _start_executing()

        ManagerdLockdown2.register_selector_callback('_XCT_testBundleReadyWithProtocolVersion:minimumVersion:',
                                                     _start_executing)
        ManagerdLockdown2.register_selector_callback('_XCT_logDebugMessage:', _show_log_message)
        ManagerdLockdown2.register_selector_callback('_XCT_didFinishExecutingTestPlan', lambda _: self.quit_event.set())

        result = ManagerdLockdown2.call('dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface',
                                        '_IDE_initiateSessionWithIdentifier:forClient:atPath:protocolVersion:',
                                        RawObj(session_identifier),
                                        str(session_identifier) + '-6722-000247F15966B083',
                                        '/Applications/Xcode.app/Contents/Developer/usr/bin/xcodebuild',
                                        RawObj(XCODE_VERSION)
                                        ).selector
        logging.info("result: %s", result)
        # launch_wda
        xctest_path = "/tmp/WebDriverAgentRunner-" + str(session_identifier).upper() + ".xctestconfiguration"
        xctest_content = archive(XCTestConfiguration({
            "testBundleURL": NSURL(None, "file://" + app_info['Path'] + "/PlugIns/WebDriverAgentRunner.xctest"),
            "sessionIdentifier": session_identifier,
        }))

        fsync = HouseArrestService(self.lockdown)
        fsync.send_command(self.bundle_id)
        for fname in fsync.read_directory("/tmp"):
            if fname.endswith(".xctestconfiguration"):
                logging.debug("remove /tmp/%s", fname)
                fsync.file_remove("/tmp/" + fname)
        fsync.set_file_contents(xctest_path, xctest_content)

        conn = InstrumentServer(self.lockdown).init()
        conn.call('com.apple.instruments.server.services.processcontrol', 'processIdentifierForBundleIdentifier:',
                  self.bundle_id)

        conn.register_undefined_callback(_callback)
        app_path = app_info['Path']
        app_container = app_info['Container']

        xctestconfiguration_path = app_container + xctest_path
        logging.info("AppPath: %s", app_path)
        logging.info("AppContainer: %s", app_container)

        app_env = {
            'CA_ASSERT_MAIN_THREAD_TRANSACTIONS': '0',
            'CA_DEBUG_TRANSACTIONS': '0',
            'DYLD_FRAMEWORK_PATH': app_path + '/Frameworks:',
            'DYLD_LIBRARY_PATH': app_path + '/Frameworks',
            'NSUnbufferedIO': 'YES',
            'SQLITE_ENABLE_THREAD_ASSERTIONS': '1',
            'WDA_PRODUCT_BUNDLE_IDENTIFIER': '',
            'XCTestConfigurationFilePath': xctestconfiguration_path,
            'XCODE_DBG_XPC_EXCLUSIONS': 'com.apple.dt.xctestSymbolicator',
            'MJPEG_SERVER_PORT': '',
            'USE_PORT': '',
        }
        if self.lockdown.ios_version > LooseVersion('11.0'):
            app_env['DYLD_INSERT_LIBRARIES'] = '/Developer/usr/lib/libMainThreadChecker.dylib'
            app_env['OS_ACTIVITY_DT_MODE'] = 'YES'
        app_options = {'StartSuspendedKey': False}
        if self.lockdown.ios_version > LooseVersion('12.0'):
            app_options['ActivateSuspended'] = True

        app_args = [
            '-NSTreatUnknownArgumentsAsOpen', 'NO',
            '-ApplePersistenceIgnoreState', 'YES'
        ]

        identifier = "launchSuspendedProcessWithDevicePath:bundleIdentifier:environment:arguments:options:"

        pid = conn.call('com.apple.instruments.server.services.processcontrol', identifier,
                        app_path, self.bundle_id, app_env, app_args, app_options).selector
        if not isinstance(pid, int):
            logging.error(f"Launch failed: {pid}")
            raise Exception("Launch failed")

        logging.info(f"Launch {self.bundle_id} pid: {pid}")

        conn.call('com.apple.instruments.server.services.processcontrol', "startObservingPid:", RawObj(pid))

        if self.quit_event:
            conn.register_selector_callback(DTXEnum.FINISHED, lambda _: self.quit_event.set())

        if self.lockdown.ios_version > LooseVersion('12.0'):
            identifier = '_IDE_authorizeTestSessionWithProcessID:'
            result = ManagerdLockdown1.call(
                'dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface',
                identifier,
                RawObj(pid)).selector
            logging.info("_IDE_authorizeTestSessionWithProcessID: %s", result)
        else:
            identifier = '_IDE_initiateControlSessionForTestProcessID:protocolVersion:'
            result = ManagerdLockdown1.call(
                'dtxproxy:XCTestManager_IDEInterface:XCTestManager_DaemonConnectionInterface',
                identifier,
                RawObj(pid), RawObj(XCODE_VERSION)).selector
            logging.info("_IDE_authorizeTestSessionWithProcessID: %s", result)

        while not self.quit_event.wait(.1):
            pass
        logging.warning("xctrunner quited")
        conn.stop()
        ManagerdLockdown2.stop()
        ManagerdLockdown1.stop()


if __name__ == '__main__':
    bundle_id = 'cn.rongcloud.autotest.xctrunner'
    xcuitest = RunXCUITest(bundle_id)
    xcuitest.start()
    time.sleep(100)
    xcuitest.close()



#!/usr/servers/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import logging
import os
import re
import sys
from optparse import OptionParser
from sys import exit
sys.path.append(os.getcwd())
from ios_device.util.lockdown import LockdownClient


TIME_FORMAT = '%H:%M:%S'


class SyslogServer(object):
    """
    View system logs
    """
    SERVICE_NAME = "com.apple.syslog_relay"

    def __init__(self, lockdown=None, udid=None, network=None,logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown if lockdown else LockdownClient(udid=udid,network=network)
        self.c = self.lockdown.start_service(self.SERVICE_NAME)

    def watch(self,log_file=None, filter=None):
        """View log
        :param logFile: full path to the log file
        :param filter: filter strings
        """
        while True:
            d = self.c.recv(4096)
            d = d.decode('utf-8')
            if filter:
                procFilter = re.compile(filter, re.IGNORECASE)
                if not procFilter.search(d):
                    continue
            s = d.strip("\n\x00\x00")
            print(s)
            if log_file:
                if isinstance(log_file,str):
                    with open(log_file, 'wt') as f:
                        f.write(d.replace("\x00", ""))
                else:
                    log_file.write(d.replace("\x00", ""))


if __name__ == "__main__":
    parser = OptionParser(usage="%prog")
    parser.add_option("-u", "--udid",
                      default=False, action="store", dest="device_udid", metavar="DEVICE_UDID",
                      help="Device udid")
    parser.add_option("-p", "--process", dest="procName", default=False,
                      help="Show process log only", type="string")
    parser.add_option("-o", "--logfile", dest="logFile", default=False,
                      help="Write Logs into specified file", type="string")
    parser.add_option("-w", "--watch-time",
                      default=False, action="store", dest="watchtime", metavar="WATCH_TIME",
                      help="watchtime")
    (options, args) = parser.parse_args()

    try:
        try:
            logging.basicConfig(level=logging.INFO)
            lckdn = LockdownClient(options.device_udid)
            syslog = SyslogServer(lockdown=lckdn)
            syslog.watch( filter=options.procName, log_file=options.logFile)
        except KeyboardInterrupt:
            print("KeyboardInterrupt caught")
            raise
        else:
            pass
    except (KeyboardInterrupt, SystemExit):
        exit()



"""
Utils
"""
import dataclasses
import math
import os

__all__ = ['DictAttrProperty', 'DictAttrFieldNotFoundError']
ROOT_DIR = os.path.abspath(os.path.dirname(__file__))

import struct

_NotSet = object()


class cached_property(object):
    def __init__(self, func):
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        return value


class DictAttrProperty:
    def __init__(self, attr, path, type=_NotSet, default=_NotSet, default_factory=_NotSet, delim='.'):
        """
        Descriptor that acts as a cached property for retrieving values nested in a dict stored in an attribute of the
        object that this :class:`DictAttrProperty` is a member of.  The value is not accessed or stored until the first
        time that it is accessed.

        As an instance attribute of a subclass of :class:`DictAttrPropertyMixin` (or any class that has
        :class:`DictAttrPropertyMeta` as its metaclass), replaces itself with the value found at the given key in that
        instance's provided attribute.  Without :class:`DictAttrPropertyMeta` as its metaclass, the value is re-computed
        each time it is accessed.

        To un-cache a value (causes the descriptor to take over again)::\n
            # >>> del instance.__dict__[attr_name]

        The :class:`ClearableCachedPropertyMixin` mixin class can be used to facilitate clearing all
        :class:`DictAttrProperty` and any similar cached properties that exist in a given object.

        :param str attr: Name of the attribute in the class that this DictAttrProperty is in that contains the dict that
          This DictAttrProperty should reference
        :param str path: The nexted key location in the dict attribute of the value that this DictAttrProperty
          represents; dict keys should be separated by ``.``, otherwise the delimiter should be provided via ``delim``
        :param callable type: Callable that accepts 1 argument; the value of this DictAttrProperty will be passed to it,
          and the result will be returned as this DictAttrProperty's value (default: no conversion)
        :param default: Default value to return if a KeyError is encountered while accessing the given path
        :param callable default_factory: Callable that accepts no arguments to be used to generate default values
          instead of an explicit default value
        :param str delim: Separator that was used between keys in the provided path (default: ``.``)
        """
        self.path = [p for p in path.split(delim) if p]
        self.path_repr = delim.join(self.path)
        self.attr = attr
        self.type = type
        self.name = '_{}#{}'.format(self.__class__.__name__, self.path_repr)
        self.default = default
        self.default_factory = default_factory

    def __set_name__(self, owner, name):
        self.name = name
        path = ''.join('[{!r}]'.format(p) for p in self.path)
        self.__doc__ = (
            "A :class:`DictAttrProperty<pypod.idevice.utils.DictAttrProperty>` that references"
            f" this {owner.__name__} instance's {self.attr}{path}"
        )

    def __get__(self, obj, cls):
        if obj is None:
            return self

        value = getattr(obj, self.attr)
        for key in self.path:
            try:
                value = value[key]
            except KeyError:
                if self.default is not _NotSet:
                    value = self.default
                    break
                elif self.default_factory is not _NotSet:
                    value = self.default_factory()
                    break
                raise DictAttrFieldNotFoundError(obj, self.name, self.attr, self.path_repr)

        if self.type is not _NotSet:
            # noinspection PyArgumentList
            value = self.type(value)
        if '#' not in self.name:
            obj.__dict__[self.name] = value
        return value


class DictAttrFieldNotFoundError(Exception):
    def __init__(self, obj, prop_name, attr, path_repr):
        self.obj = obj
        self.prop_name = prop_name
        self.attr = attr
        self.path_repr = path_repr

    def __str__(self):
        fmt = '{!r} object has no attribute {!r} ({} not found in {!r}.{})'
        return fmt.format(type(self.obj).__name__, self.prop_name, self.path_repr, self.obj, self.attr)


def kperf_data(messages):
    _list = []
    p_record = 0
    m_len = len(messages)
    if m_len % 64 != 0:
        return _list
    while p_record < m_len:
        _list.append(struct.unpack('<QLLQQQQLLQ', messages[p_record:p_record + 64]))
        p_record += 64
    return _list


def convertBytes(_bytes):
    lst = ['Bytes', 'KiB', 'MiB', 'GiB', 'TiB']
    i = 0
    if _bytes:
        i = int(math.floor(  # 舍弃小数点，取小
            math.log(_bytes, 1024)  # 求对数(对数：若 a**b = N 则 b 叫做以 a 为底 N 的对数)
        ))
    if i >= len(lst):
        i = len(lst) - 1
    return ('%.2f' + " " + lst[i]) % (_bytes / math.pow(1024, i))


class DumpDisk:

    def __init__(self):
        self.last_diskBytesRead = 1e100
        self.last_diskBytesWritten = 1e100
        self.last_diskReadOps = 0
        self.last_diskWriteOps = 0
        self.filter = ["Data Read", "Data Read/sec", "Data Written", "Data Written/sec", "Reads in", "Reads in/sec",
                       "Writes Out", "Writes Out/sec"]

    def decode(self, System):
        diskBytesRead = System.get('diskBytesRead')
        diskBytesRead_str = f"{convertBytes(diskBytesRead)}"
        diskBytesRead_qps = f"{convertBytes(diskBytesRead - self.last_diskBytesRead)}" if diskBytesRead - self.last_diskBytesRead > 0 else 0
        self.last_diskBytesRead = diskBytesRead

        diskBytesWritten = System.get('diskBytesWritten')

        diskBytesWritten_str = f"{convertBytes(diskBytesWritten)}"
        diskBytesWritten_qps = f"{convertBytes(diskBytesWritten - self.last_diskBytesWritten)}" if diskBytesWritten - self.last_diskBytesWritten > 0 else 0
        self.last_diskBytesWritten = diskBytesWritten

        diskReadOps = System.get('diskReadOps')
        diskReadOps_sec = diskReadOps - self.last_diskReadOps if diskReadOps - self.last_diskReadOps > 0 else 0
        self.last_diskReadOps = diskReadOps
        diskWriteOps = System.get('diskWriteOps')

        diskWriteOps_sec = diskWriteOps - self.last_diskWriteOps if diskWriteOps - self.last_diskWriteOps > 0 else 0
        self.last_diskWriteOps = diskWriteOps
        disk = [diskBytesRead_str, diskBytesRead_qps, diskBytesWritten_str, diskBytesWritten_qps, diskReadOps,
                diskReadOps_sec, diskWriteOps, diskWriteOps_sec]
        return dict(zip(self.filter, disk))


class DumpNetwork:

    def __init__(self):
        self.last_netBytesIn = 1e100
        self.last_netBytesOut = 1e100
        self.last_netPacketsIn = 0
        self.last_netPacketsOut = 0
        self.filter = ["Data Received", "Data Received/sec", "Data Sent", "Data Sent/sec", "Packets in",
                       "Packets in/sec",
                       "Packets Out", "Packets Out/sec"]

    def decode(self, System):
        netBytesIn = System.get('netBytesIn')
        netBytesIn_str = f"{convertBytes(netBytesIn)}"
        netBytesIn_qps = f"{convertBytes(netBytesIn - self.last_netBytesIn)}" if netBytesIn - self.last_netBytesIn > 0 else 0
        self.last_netBytesIn = netBytesIn

        netBytesOut = System.get('netBytesOut')

        netBytesOut_str = f"{convertBytes(netBytesOut)}"
        netBytesOut_qps = f"{convertBytes(netBytesOut - self.last_netBytesOut)}" if netBytesOut - self.last_netBytesOut > 0 else 0
        self.last_netBytesOut = netBytesOut

        netPacketsIn = System.get('netPacketsIn')
        netPacketsIn_sec = netPacketsIn - self.last_netPacketsIn if netPacketsIn - self.last_netPacketsIn > 0 else 0
        self.last_netPacketsIn = netPacketsIn

        netPacketsOut = System.get('netPacketsOut')
        diskWriteOps_sec = netPacketsOut - self.last_netPacketsOut if netPacketsOut - self.last_netPacketsOut > 0 else 0
        self.last_netPacketsOut = netPacketsOut

        data = [netBytesIn_str, netBytesIn_qps, netBytesOut_str, netBytesOut_qps, netPacketsOut,
                netPacketsIn_sec, netPacketsOut, diskWriteOps_sec]
        return dict(zip(self.filter, data))


class DumpMemory:

    def __init__(self):
        self.kernel_page_size = 16384  # core_profile_session_tap get kernel_page_size

    def decode(self, system: dict):
        App_Memory = convertBytes((system.get('vmIntPageCount') - system.get("vmPurgeableCount")) * self.kernel_page_size)
        Cached_Files = convertBytes((system.get('vmExtPageCount') + system.get("vmPurgeableCount")) * self.kernel_page_size)
        Compressed = convertBytes(system.get('vmCompressorPageCount') * self.kernel_page_size)
        Memory_Used = convertBytes((system.get('vmUsedCount')-system.get('vmExtPageCount')) * self.kernel_page_size)
        Wired_Memory = convertBytes(system.get("vmWireCount") * self.kernel_page_size)
        Swap_Used = convertBytes(system.get("__vmSwapUsage"))
        Free_Memory = convertBytes(system.get("vmFreeCount") * self.kernel_page_size)
        data = {"App Memory": App_Memory, "Free Memory":Free_Memory,"Cached Files": Cached_Files, "Compressed": Compressed,
                "Memory Used": Memory_Used, "Wired Memory":Wired_Memory,"Swap Used":Swap_Used}
        return data



"""
获取 app 详细列表信息
"""
import json

from ios_device.servers.dvt import DTXServer
from ios_device.servers.Instrument import InstrumentServer


def applictionListing(rpc: DTXServer):
    ret = rpc.call(
        "com.apple.instruments.server.services.device.applictionListing",
        "installedApplicationsMatching:registerUpdateToken:",
        {}, "")
    print(json.dumps(ret.selector, indent=4))
    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    applictionListing(rpc)
    rpc.stop()



import functools
import io
import sys

import click

from ios_device.cli.base import InstrumentsBase
from ios_device.cli.cli import Command, print_json
from ios_device.servers.Installation import InstallationProxyService
from ios_device.servers.crash_log import CrashLogService
from ios_device.servers.diagnostics_relay import DiagnosticsRelayService
from ios_device.servers.house_arrest import HouseArrestService
from ios_device.servers.mc_install import MCInstallService
from ios_device.servers.pcapd import PcapdService, PCAPPacketDumper
from ios_device.servers.syslog import SyslogServer
from ios_device.util import Log
from ios_device.util.lockdown import LockdownClient
from ios_device.util.usbmux import USBMux
from ios_device.util.variables import LOG

log = Log.getLogger(LOG.Mobile.value)


@click.group()
def cli():
    """ instruments cli """


@cli.group(short_help='crash report options')
def crash():
    """
    获取 crash log 相关信息
    crash report options
    """


@crash.command('delete', cls=Command)
@click.option('--name', help='crash name')
def crash_delete(udid, network, format, name):
    """ remove crash report """
    crash_server = CrashLogService(udid=udid, network=network, logger=log)
    crash_server.delete_crash(name)


@crash.command('export', cls=Command)
@click.option('--name', help='crash name')
def crash_export(udid, network, format, name):
    """ pull crash report """
    crash_server = CrashLogService(udid=udid, network=network, logger=log)
    crash_server.export_crash(name)


@crash.command('list', cls=Command)
def crash_list(udid, network, format):
    """ Show crash list """
    crash_server = CrashLogService(udid=udid, network=network, logger=log)
    crash_server.get_list()


@crash.command('shell', cls=Command)
def crash_shell(udid, network, format):
    """ Open command line mode """
    crash_server = CrashLogService(udid=udid, network=network, logger=log)
    crash_server.shell()


#######################################################################

@cli.command('sandbox', cls=Command)
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
@click.option('-a', '--access_type', default='VendDocuments', type=click.Choice(['VendDocuments', 'VendContainer']),
              help='Type of access sandbox')
def sandbox(udid, network, format, bundle_id, access_type):
    """ open an AFC shell for given bundle_id, assuming its profile is installed """
    HouseArrestService(udid=udid, network=network, logger=log).shell(bundle_id, cmd=access_type)


#######################################################################

@cli.command('devices', cls=Command)
def cmd_devices(udid, network, format):
    """ get device list """
    print_json(USBMux().get_devices(network), format)


@cli.command('deviceinfo', cls=Command)
def cmd_deviceinfo(udid, network, format):
    """ open an AFC shell for given bundle_id, assuming its profile is installed """
    device_info = LockdownClient(udid=udid, network=network).get_value()
    print_json(device_info,format=format)


#######################################################################

@cli.group(short_help='crash report options')
def profiles():
    """
    描述文件管理 例如：安装 卸载 Fiddler 证书等

    profiles & Device Management
    """


@profiles.command('list', cls=Command)
def profile_list(udid, network, format):
    """ list installed profiles """
    print_json(MCInstallService(udid=udid, network=network, logger=log).get_profile_list(), format)


@profiles.command('install', cls=Command)
@click.option('--path', type=click.File('rb'))
def profile_install(udid, network, format, path):
    """ install given profile file """
    print_json(MCInstallService(udid=udid, network=network, logger=log).install_profile(path.read()), format)


@profiles.command('remove', cls=Command)
@click.option('--name')
def profile_remove(udid, network, format, name):
    """ remove profile by name """
    print_json(MCInstallService(udid=udid, network=network, logger=log).remove_profile(name), format)


#######################################################################

@cli.command('syslog', cls=Command)
@click.option('--path', type=click.File('wt'), help='full path to the log file')
@click.option('--filter', help='filter strings')
def cmd_syslog(udid, network, format, path, filter):
    """ file management per application bundle """
    server = SyslogServer(udid=udid, network=network, logger=log)
    server.watch(log_file=path, filter=filter)


#######################################################################


@cli.group()
def apps():
    """ application options """
    pass


@apps.command('list', cls=Command)
@click.option('-u', '--user', is_flag=True, help='include user apps')
@click.option('-s', '--system', is_flag=True, help='include system apps')
def apps_list(udid, network, format, user, system):
    """ list installed apps """
    app_types = []
    if user:
        app_types.append('User')
    if system:
        app_types.append('System')
    if not app_types:
        app_types = ['User', 'System']
    print_json(InstallationProxyService(udid=udid, network=network, logger=log).get_apps(app_types),format=format)


@apps.command('uninstall', cls=Command)
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
def uninstall(udid, network, format, bundle_id):
    """ uninstall app by given bundle_id """
    print_json(InstallationProxyService(udid=udid, network=network, logger=log).uninstall(bundle_id))


@apps.command('install', cls=Command)
@click.argument('ipa_path', required=True,type=click.Path(exists=True))
def install(udid, network, format, ipa_path):
    """ install given .ipa """
    print_json(InstallationProxyService(udid=udid, network=network, logger=log).install(ipa_path))


@apps.command('upgrade', cls=Command)
@click.option('--ipa_path', type=click.Path(exists=True))
def upgrade(udid, network, format, ipa_path):
    """ install given .ipa """
    print_json(InstallationProxyService(udid=udid, network=network, logger=log).upgrade(ipa_path))


@apps.command('shell', cls=Command)
@click.option('-b', '--bundle_id', default=None,required=True, help='Process app bundleId to filter')
@click.option('-a', '--access_type', default='VendDocuments', type=click.Choice(['VendDocuments', 'VendContainer']),
              help='filter VendDocuments or VendContainer')
def shell(udid, network, format, bundle_id, access_type):
    """ open an AFC shell for given bundle_id, assuming its profile is installed """
    HouseArrestService(udid=udid, network=network, logger=log).shell(bundle_id, cmd=access_type)


@apps.command('launch', cls=Command)
@click.option('--bundle_id', default=None, help='Process app bundleId to filter')
@click.option('--app_env', default=None, help='App launch environment variable')
def cmd_launch(udid, network, format, bundle_id: str, app_env: dict):
    """
    Launch a process.
    :param bundle_id: Arguments of process to launch, the first argument is the bundle id.
    :param app_env: App launch environment variable
    """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        pid = rpc.launch_app(bundle_id=bundle_id, app_env=app_env)
        print(f'Process launched with pid {pid}')


@apps.command('kill', cls=Command)
@click.option('-p', '--pid', type=click.INT, default=None, help='Process ID to filter')
@click.option('-n', '--name', default=None, help='Process app name to filter')
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
def cmd_kill(udid, network, format, pid, name, bundle_id):
    """ Kill a process by its pid. """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        if bundle_id or name:
            pid = rpc.get_pid(bundle_id, name)
        if not pid:
            log.warn(f'The {bundle_id, name, pid} did not start')
        rpc.kill_app(pid)
        print(f'Kill {pid} ...')


#######################################################################

@cli.command('pcapd', cls=Command)
@click.argument('outfile', required=True)
def cmd_pcapd(udid, network, format, outfile):
    """ sniff device traffic

    :param outfile: output file  or (- for stdout)
    """

    if outfile == '-':
        out_file = sys.stdout.buffer
        while isinstance(out_file, io.BufferedWriter):
            out_file = out_file.detach()
    else:
        out_file = open(outfile, 'wb', 0)
    num_packets = 0
    stderr_print = functools.partial(print, file=sys.stderr)

    def packet_callback(pkt):
        nonlocal num_packets
        num_packets += 1
        stderr_print('\r{} packets captured.'.format(num_packets), end='', flush=True)

    try:
        packet_extractor = PcapdService(udid=udid, network=network)
        packet_dumper = PCAPPacketDumper(packet_extractor, out_file)
        packet_dumper.run(packet_callback)
    except KeyboardInterrupt:
        stderr_print()
        stderr_print('closing capture ...')
        out_file.close()
    except:
        stderr_print()


#######################################################################

@cli.command('battery', cls=Command)
def cmd_battery(udid, network, format):
    """ get device battery
    """
    io_registry = DiagnosticsRelayService(udid=udid, network=network, logger=log).get_battery()
    print_json(io_registry, format=format)
    update_time = io_registry.get('UpdateTime')
    voltage = io_registry.get('Voltage')
    instant_amperage = io_registry.get('InstantAmperage')
    temperature = io_registry.get('Temperature')
    current = abs(instant_amperage)
    power = current * voltage / 1000
    log.info(
        f"[Battery] time={update_time}, current={current}, voltage={voltage}, power={power}, temperature={temperature}")

#
# @information.command('battery', cls=Command)
# def cmd_battery(udid, network, format):
#     """ get device battery
#     """
#     return



"""
获取崩溃日志列表，导出崩溃日志，删除崩溃日志
"""

import logging
import os

from ios_device.servers.afc import AFCCrashLog, AFCShell
from ios_device.util.lockdown import LockdownClient


class CrashLogService(object):
    def __init__(self, lockdown=None, udid=None, network=None,logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown if lockdown else LockdownClient(udid=udid,network=network)
        self.crash_server = AFCCrashLog(lockdown=self.lockdown)

    def get_list(self):
        crash_list = self.crash_server.read_directory('/')
        print(crash_list)

    def delete_crash(self, name):
        status = self.crash_server.file_remove('//' + name)
        print(f'delete {name} status {status}')

    def export_crash(self, name):
        data = self.crash_server.get_file_contents('//' + name)
        local_crash_file = os.path.join(os.getcwd(), name)
        with open(local_crash_file, 'wb') as (fp):
            fp.write(data)
        print(f'export path {local_crash_file}')

    def shell(self):
        AFCShell(client=self.crash_server).cmdloop("开启命令模式")



import os
import sys
import time
import uuid
from datetime import datetime


from ios_device.servers.Instrument import InstrumentServer
from ios_device.util.exceptions import InstrumentRPCParseError
from ios_device.util.kperf_data import KperfData
from ios_device.util.utils import kperf_data

sys.path.append(os.getcwd())

NANO_SECOND = 1e9  # ns
MOVIE_FRAME_COST = 1 / 24


def graphics_display(rpc):
    from numpy import mean
    from numpy.core import long

    def dropped_message(res):
        print("[DROP]", res.selector, res.raw.channel_code)

    last_frame = None
    last_1_frame_cost, last_2_frame_cost, last_3_frame_cost = 0, 0, 0
    jank_count = 0
    big_jank_count = 0
    jank_time_count = 0
    mach_time_factor = 125 / 3
    frame_count = 0
    time_count = 0
    last_time = datetime.now().timestamp()
    count_time = datetime.now().timestamp()
    _list = []

    def on_graphics_message(res):
        nonlocal frame_count, last_frame, last_1_frame_cost, last_2_frame_cost, last_3_frame_cost, time_count, mach_time_factor, last_time, \
            jank_count, big_jank_count, jank_time_count, _list, count_time
        if type(res.selector) is InstrumentRPCParseError:
            for args in kperf_data(res.selector.data):
                _time, code = args[0], args[7]
                if code == 830472984:
                    # time_count = 0
                    if not last_frame:
                        last_frame = long(_time)
                    else:
                        this_frame_cost = (long(_time) - last_frame) * mach_time_factor
                        if all([last_3_frame_cost != 0, last_2_frame_cost != 0, last_1_frame_cost != 0]):
                            if this_frame_cost > mean([last_3_frame_cost, last_2_frame_cost, last_1_frame_cost]) * 2 \
                                    and this_frame_cost > MOVIE_FRAME_COST * NANO_SECOND * 2:
                                jank_count += 1
                                jank_time_count += this_frame_cost
                                if this_frame_cost > mean(
                                        [last_3_frame_cost, last_2_frame_cost, last_1_frame_cost]) * 3 \
                                        and this_frame_cost > MOVIE_FRAME_COST * NANO_SECOND * 3:
                                    big_jank_count += 1

                        last_3_frame_cost, last_2_frame_cost, last_1_frame_cost = last_2_frame_cost, last_1_frame_cost, this_frame_cost
                        time_count += this_frame_cost
                        last_frame = long(_time)
                        frame_count += 1
                # else:
                #     time_count = (datetime.now().timestamp() - count_time) * NANO_SECOND
                    # print(time_count)

                if time_count > NANO_SECOND:
                    print(
                        f"{datetime.now().timestamp() - last_time} FPS: {frame_count / time_count * NANO_SECOND} jank: {jank_count} big_jank: {big_jank_count} stutter: {jank_time_count / time_count}")
                    jank_count = 0
                    big_jank_count = 0
                    jank_time_count = 0
                    frame_count = 0
                    time_count = 0
                    count_time = datetime.now().timestamp()

                # else:
                #     last_time = datetime.now().timestamp()

    rpc.register_undefined_callback(dropped_message)
    rpc.register_channel_callback("com.apple.instruments.server.services.coreprofilesessiontap", on_graphics_message)
    # 获取mach time比例
    Kperf = KperfData(rpc)

    machTimeInfo = rpc.call("com.apple.instruments.server.services.deviceinfo", "machTimeInfo").selector
    mach_time_factor = machTimeInfo[1] / machTimeInfo[2]

    print("set", rpc.call("com.apple.instruments.server.services.coreprofilesessiontap", "setConfig:",
                          {'rp': 10,
                           'tc': [{'kdf2': {630784000, 833617920, 830472456},
                                   'tk': 3,
                                   'uuid': str(uuid.uuid4()).upper()}],
                           'ur': 500}).selector)
    print("start",
          rpc.call("com.apple.instruments.server.services.coreprofilesessiontap", "start").selector)
    time.sleep(10000)
    print("stop", rpc.call("com.apple.instruments.server.services.coreprofilesessiontap", "stop").selector)
    rpc.stop()


if __name__ == '__main__':
    rpc = InstrumentServer().init()
    graphics_display(rpc)
    rpc.stop()



"""
USBMux client that handles iDevice descovery via USB.

:author: Doug Skrypa (original: Hector Martin "marcan" <hector@marcansoft.com>)
"""
from .variables import LOG
from ..util import  Log
import select
import socket
import struct
import sys
import plistlib
from typing import Dict, Union, Optional, Tuple, Any, Mapping, List

log = Log.getLogger(LOG.USBMux.value)

from ..util.exceptions import MuxError, MuxVersionError, NoMuxDeviceFound

__all__ = ['USBMux', 'MuxConnection', 'MuxDevice', 'UsbmuxdClient']


class MuxDevice:
    def __init__(self, proto_cls, socket_path, device):
        self.device = device
        self._proto_cls = proto_cls
        self._socket_path = socket_path
        self.serial = self.device['Properties']['SerialNumber']
        self.device_id = self.device['Properties']['DeviceID']
        self.connection_type = self.device['Properties']['ConnectionType']

    def __repr__(self):
        return f'serial:{self.serial}, ConnectionType:{self.connection_type}'

    def connect(self, port):
        connector = MuxConnection(self._socket_path, self._proto_cls)
        return connector.connect(self.device_id, port)


class MuxConnection:
    def __init__(self, socketpath, protoclass):
        self.socketpath = socketpath
        if sys.platform in ('win32', 'cygwin'):
            family = socket.AF_INET
            address = ('127.0.0.1', 27015)
        else:
            family = socket.AF_UNIX
            address = self.socketpath
        self.socket = SafeStreamSocket(address, family)
        self.proto = protoclass(self.socket)
        self.pkttag = 1
        self.devices = []  # type: List[MuxDevice]

    def _getreply(self):
        resp, tag, data = self.proto.getpacket()
        return tag, data

    def _processpacket(self):

        resp, tag, data = self.proto.getpacket()
        if resp == self.proto.TYPE_DEVICE_ADD:
            self.devices.append(
                MuxDevice(
                    self.proto.__class__,
                    self.socketpath,
                    data
                )
            )
        elif resp == self.proto.TYPE_DEVICE_REMOVE:
            for dev in self.devices:
                if dev.device_id == data['DeviceID']:
                    self.devices.remove(dev)
        elif resp == self.proto.TYPE_RESULT:
            raise MuxError('Unexpected result: %d' % resp)
        else:
            raise MuxError('Invalid packet type received: %d' % resp)

    def exchange(self, req, payload=None):
        mytag = self.pkttag
        self.pkttag += 1
        self.proto.sendpacket(req, mytag, payload or {})
        recvtag, data = self._getreply()
        if recvtag != mytag:
            raise MuxError('Reply tag mismatch: expected %d, got %d' % (mytag, recvtag))
        return data

    def listen(self):
        ret = self.exchange(self.proto.TYPE_LISTEN)
        if ret['Number'] != 0:
            raise MuxError(f'Listen failed: error {ret}')

    def process(self, timeout: Optional[float] = None):
        if self.proto.connected:
            raise MuxError('Socket is connected, cannot process listener events')
        rlo, wlo, xlo = select.select([self.socket.sock], [], [self.socket.sock], timeout)
        if xlo:
            self.socket.sock.close()
            raise MuxError('Exception in listener socket')
        if rlo:
            self._processpacket()

    def connect(self, device, port) -> socket.socket:
        ret = self.exchange(
            self.proto.TYPE_CONNECT, {'DeviceID': device, 'PortNumber': ((port & 0xFF) << 8) | (port >> 8)}
        )
        if ret['Number'] != 0:
            raise MuxError('Connect failed: error %d' % ret)
        self.proto.connected = True
        return self.socket.sock

    def close(self):
        log.debug("Socket %r closed", self)
        self.socket.sock.close()

    def __exit__(self, *args):
        self.close()


class USBMux:
    def __init__(self, socket_path=None):
        socket_path = socket_path or '/var/run/usbmuxd'
        self.socketpath = socket_path
        self.listener = MuxConnection(socket_path, PlistProtocol)
        # self.listener.listen()
        self.protoclass = PlistProtocol
        self.version = 1
        self.devices = self.listener.devices  # type: List[MuxDevice]

    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.listener.close()

    def process(self, timeout: float = 0.1):
        self.listener.process(timeout)

    def find_device(self, serial=None, network=None) -> MuxDevice:
        self.get_devices(network)
        for device in self.devices:
            if serial:
                if device.device['Properties']['SerialNumber'] == serial:
                    return device
            else:
                return device
        if serial:
            raise NoMuxDeviceFound(f'Found {len(self.devices)} MuxDevice instances, but none with {serial}')
        raise NoMuxDeviceFound('No MuxDevice instances were found')

    def get_devices(self, network=None):
        payload = {
            "MessageType": "ListDevices",
            "ClientVersionString": "libusbmuxd 1.1.0",
            "kLibUSBMuxVersion": 3,
        }
        devices = self.listener.exchange('ListDevices', payload)
        for device in devices.get('DeviceList'):
            if not network and device['Properties']['ConnectionType'] == 'Network':
                continue
            self.devices.append(
                MuxDevice(
                    self.listener.proto.__class__,
                    self.socketpath,
                    device,
                )
            )
        return self.devices

    def connect(self, dev, port):
        connector = MuxConnection(self.socketpath, self.protoclass)
        return connector.connect(dev, port)


    def listen_device(self):
        self.listener.listen()
        while True:
            data = self.listener.proto.getpacket()
            yield data

class UsbmuxdClient(MuxConnection):
    def __enter__(self):
        return self

    def __exit__(self, *args):
        self.close()

    def __init__(self):
        super().__init__('/var/run/usbmuxd', PlistProtocol)

    def get_pair_record(self, udid):
        tag = self.pkttag
        self.pkttag += 1
        payload = {'PairRecordID': udid,
                   'kLibUSBMuxVersion': 3}
        self.proto.sendpacket('ReadPairRecord', tag, payload)
        _, recvtag, data = self.proto.getpacket()
        if recvtag != tag:
            raise MuxError('Reply tag mismatch: expected %d, got %d' % (tag, recvtag))
        pair_record = data.get('PairRecordData')
        if not pair_record:
            return None
        pair_record = plistlib.loads(pair_record)
        return pair_record

    def delete_pair_record(self, udid):

        tag = self.pkttag
        self.pkttag += 1
        payload = {'PairRecordID': udid}
        self.proto.sendpacket('DeletePairRecord', tag, payload)
        _, recvtag, data = self.proto.getpacket()
        if recvtag != tag:
            raise MuxError('Reply tag mismatch: expected %d, got %d' % (tag, recvtag))
        return data

    def save_pair_record(self, udid, pair_record, devid):
        tag = self.pkttag
        self.pkttag += 1
        payload = {'PairRecordID': udid,
                   "PairRecordData": plistlib.dumps(pair_record),
                   "DeviceID": devid,
                   }
        self.proto.sendpacket('SavePairRecord', tag, payload)
        _, recvtag, data = self.proto.getpacket()
        if recvtag != tag:
            raise MuxError('Reply tag mismatch: expected %d, got %d' % (tag, recvtag))
        return data

    def read_system_buid(self):
        tag = self.pkttag
        self.pkttag += 1
        payload = {'kLibUSBMuxVersion': 3}
        self.proto.sendpacket('ReadBUID', tag, payload)
        _, recvtag, data = self.proto.getpacket()
        if recvtag != tag:
            raise MuxError('Reply tag mismatch: expected %d, got %d' % (tag, recvtag))
        return data['BUID']



class BinaryProtocol:
    TYPE_RESULT = 1
    TYPE_CONNECT = 2
    TYPE_LISTEN = 3
    TYPE_DEVICE_ADD = 4
    TYPE_DEVICE_REMOVE = 5
    TYPE_DEVICE_LIST = 6
    VERSION = 0

    def __init__(self, sock):
        self.socket = sock
        self.connected = False

    def _pack(self, req: int, payload: Optional[Mapping[str, Any]]):
        if req == self.TYPE_CONNECT:
            connect_data = b'\x00\x00'
            return struct.pack('IH', payload['DeviceID'], payload['PortNumber']) + connect_data
        elif req == self.TYPE_LISTEN:
            return b''
        else:
            raise ValueError('Invalid outgoing request type %d' % req)

    def _unpack(self, resp: int, payload: bytes) -> Dict[str, Any]:
        if resp == self.TYPE_RESULT:
            return {'Number': struct.unpack('I', payload)[0]}
        elif resp == self.TYPE_DEVICE_ADD:
            devid, usbpid, serial, pad, location = struct.unpack('IH256sHI', payload)
            serial = serial.split(b'\0')[0]
            return {
                'DeviceID': devid,
                'Properties': {
                    'LocationID': location,
                    'SerialNumber': serial,
                    'ProductID': usbpid
                }
            }
        elif resp == self.TYPE_DEVICE_REMOVE:
            devid = struct.unpack('I', payload)[0]
            return {'DeviceID': devid}
        else:
            raise MuxError('Invalid incoming response type %d' % resp)

    def sendpacket(self, req: int, tag: int, payload: Union[Mapping[str, Any], bytes, None] = None):
        payload = self._pack(req, payload or {})
        if self.connected:
            raise MuxError('Mux is connected, cannot issue control packets')
        length = 16 + len(payload)
        data = struct.pack('IIII', length, self.VERSION, req, tag) + payload
        log.debug(f'发送 Plist byte: {data}')
        self.socket.send(data)

    def getpacket(self) -> Tuple[int, int, Union[Dict[str, Any], bytes]]:
        if self.connected:
            raise MuxError('Mux is connected, cannot issue control packets')
        dlen = self.socket.recv(4)
        dlen = struct.unpack('I', dlen)[0]
        body = self.socket.recv(dlen - 4)
        version, resp, tag = struct.unpack('III', body[:0xc])
        if version != self.VERSION:
            raise MuxVersionError('Version mismatch: expected %d, got %d' % (self.VERSION, version))
        payload = self._unpack(resp, body[0xc:])
        return resp, tag, payload


class PlistProtocol(BinaryProtocol):
    TYPE_RESULT = 'Result'
    TYPE_CONNECT = 'Connect'
    TYPE_LISTEN = 'Listen'
    TYPE_DEVICE_ADD = 'Attached'
    TYPE_DEVICE_REMOVE = 'Detached'  # ???
    TYPE_DEVICE_LIST = 'ListDevices'
    TYPE_PLIST = 8
    VERSION = 1

    def _pack(self, req: int, payload: bytes) -> bytes:
        return payload

    def _unpack(self, resp: int, payload: bytes) -> bytes:
        return payload

    def sendpacket(self, req, tag, payload: Optional[Mapping[str, Any]] = None):
        payload = payload or {}
        payload['ClientVersionString'] = 'qt4i-usbmuxd'
        if isinstance(req, int):
            req = [self.TYPE_CONNECT, self.TYPE_LISTEN, self.TYPE_DEVICE_LIST][req - 2]
        payload['MessageType'] = req
        payload['ProgName'] = 'tcprelay'
        log.debug(f'发送 Plist: {payload}')
        wrapped_payload = plistlib.dumps(payload)
        super().sendpacket(self.TYPE_PLIST, tag, wrapped_payload)

    def getpacket(self):
        resp, tag, payload = super().getpacket()
        log.debug(f'接收 Plist byte: {payload}')
        if resp != self.TYPE_PLIST:
            raise MuxError('Received non-plist type %d' % resp)
        payload = plistlib.loads(payload)
        log.debug(f'接收 Plist: {payload}')
        return payload.get('MessageType', ''), tag, payload


class SafeStreamSocket:
    def __init__(self, address, family):
        self.sock = socket.socket(family, socket.SOCK_STREAM)
        self.sock.connect(address)

    def send(self, msg):
        totalsent = 0
        while totalsent < len(msg):
            sent = self.sock.send(msg[totalsent:])
            if sent == 0:
                raise MuxError('socket connection broken')
            totalsent = totalsent + sent

    def recv(self, size):
        msg = b''
        while len(msg) < size:
            chunk = self.sock.recv(size - len(msg))
            empty_chunk = b''
            if chunk == empty_chunk:
                raise MuxError('socket connection broken')
            msg = msg + chunk
        return msg



#!/usr/servers/env python
# -*- coding: utf8 -*-
#
# $Id$
#
# Copyright (c) 2012-2014 "dark[-at-]gotohack.org"
#
# This file is part of pymobiledevice
#
# pymobiledevice is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import logging
import os
import re
import sys
import time
from optparse import OptionParser
from sys import exit
sys.path.append(os.getcwd())

from ios_device.util.lockdown import LockdownClient

from six import PY3

TIME_FORMAT = '%H:%M:%S'


class Syslog(object):
    '''
    View system logs
    '''

    def __init__(self, lockdown=None, udid=None, logger=None):
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown if lockdown else LockdownClient(udid=udid)
        self.c = self.lockdown.start_service("com.apple.syslog_relay")

    def watch(self, watchtime=None, logFile=None, procName=None):
        """View log
        :param watchtime: time (seconds)
        :type watchtime: int
        :param logFile: full path to the log file
        :type logFile: str
        :param procName: process name
        :type proName: str
        """
        begin = time.strftime(TIME_FORMAT)
        while True:
            d = self.c.recv(4096)
            if PY3:
                d = d.decode('utf-8')
            if procName:
                procFilter = re.compile(procName, re.IGNORECASE)
                if len(d.split(" ")) > 4 and not procFilter.search(d):
                    continue
            s = d.strip("\n\x00\x00")
            # self.logger.info(s)
            print(s)
            if logFile:
                with open(logFile, 'a') as f:
                    f.write(d.replace("\x00", ""))
            if watchtime:
                now = self.time_match(s[7:15])
                if now:
                    time_spend = self.time_caculate(str(begin), now)
                    if time_spend > watchtime:
                        break

    def time_match(self, str_time):
        """
        Determine if the time format matches
        """
        pattern = re.compile(r'\d{2}:\d{2}:\d{2}')
        match = pattern.match(str_time)
        if match:
            return str_time
        else:
            return False

    def time_caculate(self, a, b):
        """
        Calculate the time difference between two strings
        """
        time_a = int(a[6:8]) + 60 * int(a[3:5]) + 3600 * int(a[0:2])
        time_b = int(b[6:8]) + 60 * int(b[3:5]) + 3600 * int(b[0:2])
        return time_b - time_a


if __name__ == "__main__":
    parser = OptionParser(usage="%prog")
    parser.add_option("-u", "--udid",
                      default=False, action="store", dest="device_udid", metavar="DEVICE_UDID",
                      help="Device udid")
    parser.add_option("-p", "--process", dest="procName", default=False,
                      help="Show process log only", type="string")
    parser.add_option("-o", "--logfile", dest="logFile", default=False,
                      help="Write Logs into specified file", type="string")
    parser.add_option("-w", "--watch-time",
                      default=False, action="store", dest="watchtime", metavar="WATCH_TIME",
                      help="watchtime")
    (options, args) = parser.parse_args()

    try:
        try:
            logging.basicConfig(level=logging.INFO)
            lckdn = LockdownClient(options.device_udid)
            syslog = Syslog(lockdown=lckdn)
            syslog.watch(watchtime=int(options.watchtime), procName=options.procName, logFile=options.logFile)
        except KeyboardInterrupt:
            print("KeyboardInterrupt caught")
            raise
        else:
            pass


    except (KeyboardInterrupt, SystemExit):
        exit()



import enum
import queue
import typing
from collections import defaultdict
from threading import Thread, Event

from ..util import logging, Log
from ..util.dtx_msg import DTXMessage, MessageAux, dtx_message_header, object_to_aux
from ..util.exceptions import MuxError
from ..util.variables import LOG

log = Log.getLogger(LOG.Instrument.value)


class DTXEnum(str, enum.Enum):
    FINISHED = "finished:"


class DTXClient:

    def __init__(self):
        self._dtx_manager = {}

    def recv(self, client, length) -> bytes:

        buf = bytearray()
        while len(buf) < length:
            chunk = client.recv(length - len(buf))
            if not chunk:
                raise MuxError("socket connection close")
            buf.extend(chunk)
        log.debug(f'接收 DTX buf: {buf}')
        return buf

    def send_dtx(self, client, dtx):
        buffer = bytes(dtx)
        log.debug(f'发送 DTX: {buffer}')
        return client.send(buffer)

    def recv_dtx(self, client):
        """
        :param client:
        :param timeout:  (s)
        :return:
        """
        payload = bytearray()
        while True:

            header_buffer = self.recv(client, dtx_message_header.sizeof())
            if not header_buffer:
                return None
            header = dtx_message_header.parse(header_buffer)

            if header.fragment_id == 0:
                if header.channel not in self._dtx_manager:
                    self._dtx_manager[header.channel] = (header_buffer, payload)

                if header.fragment_count > 1:
                    continue

            body_buffer = self.recv(client, header.payload_length)
            if not body_buffer:
                break
            self._dtx_manager.get(header.channel)[1].extend(body_buffer)

            if header.fragment_id == header.fragment_count - 1:
                break

        data = self._dtx_manager.get(header.channel)

        self._dtx_manager.pop(header.channel)
        return DTXMessage.decode(data[0], data[1])


class DTXServer:

    def __init__(self, lockdown=None, udid=None):
        self.udid = udid
        self.lockdown = lockdown
        self._cli = None
        self._recv_thread = None
        self._running = False
        self._callbacks = {}
        self._undefined_callback = None
        self._channel_callbacks = {}
        self._channels = {}
        self._receiver_exiting = False
        self._published_capabilities = None
        self._reply_queues = defaultdict(queue.Queue)
        self._next_identifier = 1
        self._client = DTXClient()
        self.done = Event()
        self.register()

    def register(self):
        def _notifyOfPublishedCapabilities(res):
            self.done.set()
            self._published_capabilities = res.auxiliaries

        self.register_selector_callback("_notifyOfPublishedCapabilities:", _notifyOfPublishedCapabilities)

    def init(self, _cli=None):
        """ 继承类
        初始化 servers 服务:
        :return: bool 是否成功
        """
        self._cli = _cli
        self._start()
        return self

    def _start(self):
        """
        启动 servers 服务, 此接口用户无需调用, 用户使用 init 接口
        :return: bool 是否成功
        """
        if self._running:
            return True
        self._running = True
        self._recv_thread = Thread(target=self._receiver, name="InstrumentServer")
        self._recv_thread.start()
        while not self.done.wait(5):
            logging.debug("[WARN] timeout waiting capabilities")
            return False
        return True

    def stop(self):
        """
        停止 servers 服务
        :return: 无返回值
        """
        self._running = False
        if self._recv_thread:
            self._recv_thread = None
        if self._cli:
            self._cli.close()
            self._cli = None

    def _run_callbacks(self, event_name, data):
        """
        Returns:
            if called
        """
        func = self._callbacks.get(event_name)
        if func:
            func(data)
            return True

    def register_selector_callback(self, selector: str, callback: typing.Callable):

        """
        :param selector:
        :param callback:
        """
        self._callbacks[selector] = callback

    def register_channel_callback(self, channel: str, callback: typing.Callable):
        """
        Return channel messages
        :param channel:
        :param callback
        """
        log.info(f'set {channel} callback ...')
        channel_id = self.make_channel(channel)
        self._channel_callbacks[channel_id] = callback

    def register_undefined_callback(self, callback: typing.Callable):
        """
        Returns all undefined messages
        :param callback
        """
        self._undefined_callback = callback

    def make_channel(self, channel: str):
        if channel in self._channels:
            return self._channels[channel]
        channel_id = len(self._channels) + 1
        self._call(True, 0, "_requestChannelWithCode:identifier:", channel_id, channel)
        self._channels[channel] = channel_id
        return channel_id

    def call(self, channel: str, selector: str, *auxiliaries):
        channel_id = self.make_channel(channel)
        ret = self._call(True, channel_id, selector, *auxiliaries)
        return ret

    def _reply_ack(self, data):
        reply = DTXMessage()
        reply._channel_code = data.channel_code
        reply._identifier = data.identifier
        reply._conversation_index = data.conversation_index + 1
        reply._flags = 0x3
        reply._selector = b'\00' * 16
        self._client.send_dtx(self._cli, reply)

    def wait_reply(self, message_id: int, timeout=30.0) -> DTXMessage:
        ret = self._reply_queues[message_id].get(timeout=timeout)
        if ret is None:
            raise MuxError("connection closed")
        return ret

    def _call(self, sync: bool, channel_id: int, selector: str, *auxiliaries):
        """
        :param sync: 是否回调
        :param channel_id: 通道标识
        :param selector: 请求方法名称，method name
        :param auxiliaries:
        :return:
        """
        identifier = self._next_identifier
        dtx = DTXMessage()
        dtx._identifier = identifier
        dtx._channel_code = channel_id
        dtx._selector = selector
        dtx._expects_reply = sync
        aux = MessageAux()
        dtx.auxiliaries = aux
        self._next_identifier += 1
        for arg in auxiliaries:
            object_to_aux(arg, aux)
        self._client.send_dtx(self._cli, dtx)
        if sync:
            ret = self.wait_reply(identifier)
            return ret

    def _receiver(self):
        try:
            while self._running:
                dtx = self._client.recv_dtx(self._cli)
                if '_channelCanceled:' in str(dtx.selector):
                    self._cli.close()
                if dtx.conversation_index == 1:
                    self._reply_queues[dtx.identifier].put(dtx)
                elif (2 ** 32 - dtx.channel_code) in self._channel_callbacks:
                    self._channel_callbacks[(2 ** 32 - dtx.channel_code)](dtx)
                else:
                    selector = dtx.selector
                    if isinstance(selector, str) and selector in self._callbacks:
                        self._callbacks[selector](dtx)
                    elif self._undefined_callback:
                        self._undefined_callback(dtx)
                    if dtx.expects_reply:
                        self._reply_ack(dtx)

        except MuxError as E:
            log.warn(E)
        except Exception as E:
            log.exception(E)
        finally:
            self._run_callbacks(DTXEnum.FINISHED, None)
            self.stop()



"""
@Date    : 2021-03-05
@Author  : liyachao
"""

import select
import time
from threading import Thread

from ios_device.util.usbmux import USBMux
from six import PY3
from six.moves import socketserver


class SocketRelay(object):
    def __init__(self, a, b, maxbuf=65535):
        self.a = a
        self.b = b
        self.atob = ""
        self.btoa = ""
        if PY3:
            self.atob = b""
            self.btoa = b""
        self.maxbuf = maxbuf

    def handle(self):
        while True:
            rlist = []
            wlist = []
            xlist = [self.a, self.b]
            if self.atob:
                wlist.append(self.b)
            if self.btoa:
                wlist.append(self.a)
            if len(self.atob) < self.maxbuf:
                rlist.append(self.a)
            if len(self.btoa) < self.maxbuf:
                rlist.append(self.b)
            rlo, wlo, xlo = select.select(rlist, wlist, xlist)
            if xlo:
                return
            if self.a in wlo:
                n = self.a.send(self.btoa)
                self.btoa = self.btoa[n:]
            if self.b in wlo:
                n = self.b.send(self.atob)
                self.atob = self.atob[n:]
            if self.a in rlo:
                s = self.a.recv(self.maxbuf - len(self.atob))
                if not s:
                    return
                self.atob += s
            if self.b in rlo:
                s = self.b.recv(self.maxbuf - len(self.btoa))
                if not s:
                    return
                self.btoa += s


class TCPRelay(socketserver.BaseRequestHandler):

    def handle(self):
        mux = self.server.mux
        # print("Waiting for devices...")
        mux.process(0.1)
        device = self.server.device
        dsock = device.connect(self.server.rport)
        lsock = self.request
        try:
            fwd = SocketRelay(dsock, lsock, self.server.bufsize * 1024)
            fwd.handle()
        finally:
            dsock.close()
            lsock.close()


class TCPServer(socketserver.TCPServer):
    allow_reuse_address = True


class ThreadedTCPServer(socketserver.ThreadingMixIn, TCPServer):
    pass


class ForwardPorts(Thread):
    def __init__(self, pair_ports, device_id=None, threaded=True, bufsize=128):
        """
        iOS真机设备的端口转发
        :param pair_ports: list 端口对的数组，每对端口中前一个代表远程端口，后一个代表本地端口，例如：["8100:8100", "8200:8200"]
        :param udid:
        :param threaded:
        :param bufsize:
        """
        super().__init__()
        self.pair_ports = pair_ports
        self.device_id = device_id
        self.threaded = threaded
        self.bufsize = bufsize
        self._server = None

    def forward_ports(self):

        if self.threaded:
            serverclass = ThreadedTCPServer
        else:
            serverclass = TCPServer
        try:
            for pair_port in self.pair_ports:
                rport, lport = pair_port.split(":")
                rport = int(rport)
                lport = int(lport)
                self._server = serverclass(("localhost", lport), TCPRelay)
                self._server.rport = rport
                self._server.bufsize = self.bufsize
                self._server.udid = self.device_id
                mux = USBMux()
                mux.process()
                self._server.mux = mux
                self._server.device = mux.find_device(serial=self.device_id)
                self._server.serve_forever()
        except Exception as e:
            self.stop()
            print(e)
            exit(1)

    def run(self) -> None:
        self.forward_ports()

    def stop(self):
        time.sleep(0.5)
        if self._server:
            self._server.shutdown()


if __name__ == '__main__':
    c = ForwardPorts(["8200:8200"])
    c.start()
    # c.stop()



import dataclasses
import json
import threading
import uuid
from copy import deepcopy
from datetime import datetime
from distutils.version import LooseVersion

import click

from ios_device.cli.base import InstrumentsBase
from ios_device.cli.cli import Command, print_json
from ios_device.util import Log, api_util
from ios_device.util.exceptions import InstrumentRPCParseError
from ios_device.util.kc_data import kc_data_parse
from ios_device.util.utils import DumpDisk, DumpNetwork, DumpMemory
from ios_device.util.variables import LOG

log = Log.getLogger(LOG.Instrument.value)


@click.group()
def cli():
    """ instruments cli """


@cli.group(short_help='run instruments service')
def instruments():
    """
    运行 instruments 组件相关服务

    run instruments service
    """


@instruments.command('runningProcesses', cls=Command, short_help='Show running process list')
def cmd_running_processes(udid, network, format):
    """
    显示正在运行的进程信息

    Show running process list
     """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        processes = rpc.device_info.runningProcesses()
        print_json(processes, format)


@instruments.command('applist', cls=Command)
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
def cmd_application(udid, network, format, bundle_id):
    """ Show application list """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        apps = rpc.application_listing(bundle_id)
        print_json(apps, format)


@instruments.command('kill', cls=Command)
@click.option('-p', '--pid', type=click.INT, default=None, help='Process ID to filter')
@click.option('-n', '--name', default=None, help='Process app name to filter')
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
def cmd_kill(udid, network, format, pid, name, bundle_id):
    """ Kill a process by its pid. """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        if bundle_id or name:
            pid = rpc.get_pid(bundle_id, name)
        if not pid:
            log.error(f'The pid: {pid} did not start. Try "-h or --help" for help')
            return
        rpc.kill_app(pid)
        print(f'Kill {pid} ...')


@instruments.command('launch', cls=Command)
@click.option('--bundle_id', default=None, help='Process app bundleId to filter')
@click.option('--app_env', default=None, help='App launch environment variable')
def cmd_launch(udid, network, format, bundle_id: str, app_env: dict):
    """
    Launch a process.
    :param bundle_id: Arguments of process to launch, the first argument is the bundle id.
    :param app_env: App launch environment variable
    """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        pid = rpc.launch_app(bundle_id=bundle_id, app_env=app_env)
        print(f'Process launched with pid {pid}')


@instruments.group('information')
def information():
    """ System information. """


@information.command('system', cls=Command)
def cmd_information_system(udid, network, format):
    """ Print system information. """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        print_json(rpc.device_info.systemInformation(), format)


@information.command('hardware', cls=Command)
def cmd_information_hardware(udid, network, format):
    """ Print hardware information. """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        print_json(rpc.device_info.hardwareInformation(), format)


@information.command('network', cls=Command)
def cmd_information_network(udid, network, format):
    """ Print network information. """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        print_json(rpc.device_info.networkInformation(), format)


@instruments.command('xcode_energy', cls=Command)
@click.option('-p', '--pid', type=click.INT, default=None, help='Process ID to filter')
@click.option('-n', '--name', default=None, help='Process app name to filter')
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
def cmd_xcode_energy(udid, network, pid, name, bundle_id, format):
    """ Print process about current network activity.  """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        if bundle_id or name:
            pid = rpc.get_pid(bundle_id, name)
        if not pid:
            log.error(f'The pid: {pid} did not start. Try "--help" for help')
            return
        rpc.xcode_energy(pid)


@instruments.command('network_process', cls=Command)
@click.option('-p', '--pid', type=click.INT, default=None, help='Process ID to filter')
@click.option('-n', '--name', default=None, help='Process app name to filter')
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
def cmd_network_process(udid, network, pid, name, bundle_id, format):
    """ Print process about current network activity.  """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        if bundle_id or name:
            pid = rpc.get_pid(bundle_id, name)
        if not pid:
            log.error(f'The pid: {pid} did not start. Try "--help" for help')
            return
        rpc.xcode_network(pid)


@instruments.command('networking', cls=Command)
def cmd_networking(udid, network, format):
    """ Print information about current network activity. """

    def _callback(res):
        api_util.network_caller(res, print_json)

    with InstrumentsBase(udid=udid, network=network) as rpc:
        rpc.networking(_callback)


@instruments.command('sysmontap', cls=Command)
@click.option('-t', '--time', type=click.INT, default=1000, help='Output interval time (ms)')
@click.option('-p', '--pid', type=click.INT, default=None, help='Process ID to filter')
@click.option('-n', '--name', default=None, help='Process app name to filter')
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter.Omit show all')
@click.option('--processes', is_flag=True, help='Only output process information')
@click.option('--sort', help='Process field sorting')
@click.option('--proc_filter', help='Process param to filter split by ",". Omit show all')
@click.option('--sys_filter', help='System param to filter split by ",". Omit show all')
def cmd_sysmontap(udid, network, format, time, pid, name, bundle_id, processes, sort, proc_filter,
                  sys_filter):
    """ Get performance data """

    def on_callback_message(res):
        if isinstance(res.selector, list):
            data = deepcopy(res.selector)
            processes_data = {}
            for index, row in enumerate(res.selector):
                if 'Processes' in row:
                    data[index]['Processes'] = {}
                    for _pid, process in row['Processes'].items():
                        process_attributes = dataclasses.make_dataclass('SystemProcessAttributes',
                                                                        proc_filter or rpc.process_attributes)
                        attrs = process_attributes(*process)
                        if pid and pid != _pid:
                            continue
                        if name and attrs.name != name:
                            continue
                        if processes:
                            processes_data[f'{attrs.name}'] = attrs.__dict__
                            continue
                        data[index]['Processes'][f'{attrs.name}'] = attrs.__dict__
                    data[index]['Processes'] = sorted(data[index]['Processes'].items(),
                                                      key=lambda d: d[1].get(sort, 0),
                                                      reverse=True)

                if 'System' in row:
                    if 'SystemAttributes' in data[index]:
                        del data[index]['SystemAttributes']
                    if 'ProcessesAttributes' in data[index]:
                        del data[index]['ProcessesAttributes']
                    data[index]['System'] = dict(zip(rpc.system_attributes, row['System']))
            if processes:
                processes_data = sorted(processes_data.items(), key=lambda d: d[1].get(sort, 0) or 0,
                                        reverse=True)
                print_json(processes_data, format)
            else:
                print_json(data, format)

    with InstrumentsBase(udid=udid, network=network) as rpc:

        if proc_filter:
            data = rpc.device_info.sysmonProcessAttributes()
            proc_filter = proc_filter.split(',')
            proc_filter.extend(['name', 'pid'])
            proc_filter = list(set(proc_filter))
            for proc in proc_filter:
                if proc not in data:
                    log.warn(f'{proc_filter} value：{proc} not in {data}')
                    return
            rpc.process_attributes = proc_filter

        if sys_filter:
            data = rpc.device_info.sysmonSystemAttributes()
            sys_filter = sys_filter.split(',')
            for sys in sys_filter:
                if sys not in data:
                    log.warn(f'{sys_filter} value：{sys} not in {data}')
                    return
            rpc.system_attributes = sys_filter

        if bundle_id:
            app = rpc.application_listing(bundle_id)
            name = app.get('ExecutableName')
        rpc.sysmontap(on_callback_message, time)


@instruments.command('monitor', cls=Command)
@click.option('--filter', default="all", type=click.Choice(["all", 'disk', 'network', 'memory', 'cpu']), help='')
def cmd_monitor(udid, network, format, filter: str):
    """ Get monitor performance data """
    disk = DumpDisk()
    Network = DumpNetwork()
    Memory = DumpMemory()

    def on_callback_message(res):
        data = {}
        SystemCPUUsage = {}
        if isinstance(res.selector, list):
            for index, row in enumerate(res.selector):
                if 'System' in row:
                    data = dict(zip(rpc.system_attributes, row['System']))
                if "SystemCPUUsage" in row:
                    SystemCPUUsage = row["SystemCPUUsage"]
            if 'disk' == filter.lower():
                print("Disk    >>", disk.decode(data))
            if 'network' == filter.lower():
                print("Network >>", Network.decode(data))
            if 'memory' == filter.lower():
                print("Memory  >>", Memory.decode(data))
            if 'cpu' == filter.lower():
                print("CPU     >>", SystemCPUUsage)
            if "all" == filter.lower():
                print("Memory  >>", Memory.decode(data))
                print("Network >>", Network.decode(data))
                print("Disk    >>", disk.decode(data))
                print("CPU     >>", SystemCPUUsage)

    with InstrumentsBase(udid=udid, network=network) as rpc:
        rpc.process_attributes = ['name', 'pid']
        rpc.sysmontap(on_callback_message)


@instruments.group()
def condition():
    """
    Set system running condition
    """


@condition.command('get', cls=Command)
def cmd_get_condition_inducer(udid, network, format):
    """ get aLL condition inducer configuration
    """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        ret = rpc.get_condition_inducer()
        print_json(ret, format)


@condition.command('set', cls=Command)
@click.option('-c', '--condition_id', default=None, help='Process app bundleId to filter')
@click.option('-p', '--profile_id', default='', help='start wda port')
def cmd_set_condition_inducer(udid, network, format, condition_id, profile_id):
    """ set condition inducer
    """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        ret = rpc.set_condition_inducer(condition_id, profile_id)
        print_json(ret, format)


@instruments.command('xcuitest', cls=Command)
@click.option('-b', '--bundle_id', default=None, help='Process app bundleId to filter')
@click.option('-p', '--port', default='', help='start wda port')
def cmd_xcuitest(udid, network, format, bundle_id, port):
    """ Run XCTest required WDA installed.
    """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        rpc.xctest(bundle_id, port)


@instruments.command('fps', cls=Command)
@click.option('-t', '--time', type=click.INT, default=1000, help='Output interval time (ms)')
def cmd_graphics(udid, network, format, time):
    """ Get graphics fps
    """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        def on_callback_message(res):
            data = res.selector
            print_json({"currentTime": str(datetime.now()), "fps": data['CoreAnimationFramesPerSecond']}, format)

        rpc.graphics(on_callback_message, time)


@instruments.command('notifications', cls=Command)
def cmd_notifications(udid, network, format):
    """Get mobile notifications
    """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        def on_callback_message(res):
            print_json(res.auxiliaries, format)

        rpc.mobile_notifications(on_callback_message)


@instruments.command('stackshot', cls=Command)
@click.option('--out', type=click.File('w'), default=None)
def stackshot(udid, network, format, out):
    """ Dump stack snapshot information. """
    with InstrumentsBase(udid=udid, network=network) as rpc:

        if rpc.lock_down.ios_version < LooseVersion('12.0'):
            log.warn('The interface requires iOS version 12+')
            return
        stopSignal = threading.Event()

        def on_callback_message(res):
            if isinstance(res.selector, InstrumentRPCParseError):
                buf = res.selector.data
                if buf.startswith(b'\x07X\xa2Y'):
                    stopSignal.set()
                    kc_data = kc_data_parse(buf)
                    if out is not None:
                        json.dump(kc_data, out, indent=4)
                        log.info(f'Successfully dump stackshot to {out.name}')
                    else:
                        print_json(kc_data, format)

        rpc.core_profile_session(on_callback_message, stopSignal)


@instruments.command('core_profile', cls=Command)
@click.option('--pid', type=click.INT, default=None, help='Process ID to filter')
@click.option('--process-name',default=None, help='Process name to filter')
def stackshot(udid, network, format,pid,process_name):
    """ Dump stack snapshot information. """
    with InstrumentsBase(udid=udid, network=network) as rpc:
        config={
                 'tc': [{
                     'csd': 128,
                     'kdf2': {0xffffffff},
                     'ta': [[3], [0], [2], [1, 1, 0]],
                     'tk': 3,
                     'uuid': str(uuid.uuid4()),
                 }],
                 'rp': 100,
                 'bm': 0,
             }
        rpc.core_profile(config,pid,process_name)


"""
Plist Service - handles parsing and formatting plist content
"""
from .exceptions import MuxError
from ..util import  Log
import plistlib
import re
import ssl
import struct
from socket import socket
from typing import Optional, Dict, Any

from .usbmux import USBMux, MuxDevice

__all__ = ['PlistService']
log = Log.getLogger(__name__)
HARDWARE_PLATFORM_SUB = re.compile(r'[^\w<>/ \-_0-9\"\'\\=.?!+]+').sub


class PlistService:
    def __exit__(self, *args):
        self.close()

    def __init__(
            self,
            port: int = 62078,
            udid: Optional[str] = None,
            device: Optional[MuxDevice] = None,
            ssl_file: Optional[str] = None,
            network=None
    ):
        self.port = port
        self.device = device or USBMux().find_device(udid, network)
        log.debug(f'Connecting to device: {self.device.serial}')
        self.sock = self.device.connect(port)  # type: socket
        if ssl_file:
            self.ssl_start(ssl_file, ssl_file)

    def ssl_start(self, keyfile, certfile):
        self.sock = ssl.wrap_socket(self.sock, keyfile, certfile)

    def send(self, msg):
        totalsent = 0
        while totalsent < len(msg):
            sent = self.sock.send(msg[totalsent:])
            if sent == 0:
                raise MuxError('socket connection broken')
            totalsent = totalsent + sent

    def recv(self, length=4096, timeout=-1):
        try:
            if timeout > 0:
                self.sock.settimeout(timeout)
            buf = self.sock.recv(length)
            return buf
        except Exception as E:
            return b''

    def close(self):
        self.sock.close()

    def recv_exact(self, to_read) -> bytes:
        buffer = bytearray(to_read)
        view = memoryview(buffer)
        while view:
            received = self.sock.recv_into(view, to_read)
            if received:
                view = view[received:]
                to_read -= received
            else:
                break
        return buffer

    def recv_plist(self) -> Optional[Dict[str, Any]]:
        resp = self.recv_exact(4)
        if not resp or len(resp) != 4:
            return None
        payload = self.recv_exact(struct.unpack('>L', resp)[0])
        log.debug(f'接收 Plist byte: {payload}')

        if not payload:
            return None
        if payload.startswith(b'bplist00') or payload.startswith(b'<?xml'):
            data = plistlib.loads(payload)
            log.debug(f'接收 Plist data: {data}')
            return data
        else:
            raise ValueError('Received invalid data: {}'.format(payload[:100].decode('hex')))

    def send_plist(self, data):
        log.debug(f'发送 Plist: {data}')
        payload = plistlib.dumps(data)
        payload_len = struct.pack('>L', len(payload))
        log.debug(f'发送 Plist byte: {payload_len+payload}')
        return self.sock.send(payload_len + payload)

    def plist_request(self, request):
        self.send_plist(request)
        return self.recv_plist()



import logging
import os
import typing


class MobileImageMounter(object):

    SERVICE_NAME = 'com.apple.mobile.mobile_image_mounter'

    def __init__(self, lockdown=None, udid=None, logger=None,service=None):
        from ..util.lockdown import LockdownClient
        self.logger = logger or logging.getLogger(__name__)
        self.lockdown = lockdown if lockdown else LockdownClient(udid=udid)
        self.service = service if service else self.lockdown.start_service(self.SERVICE_NAME)

        if not self.lockdown:
            raise Exception("Unable to start lockdown")
        if not self.service:
            raise Exception("installation_proxy init error : Could not start com.apple.mobile.mobile_image_mounter")

    def lookup(self, image_type="Developer") -> typing.List[bytes]:
        """
        Check image signature
        """
        ret = self.service.plist_request({
            "Command": "LookupImage",
            "ImageType": image_type,
        })
        if 'Error' in ret:
            raise Exception(ret['Error'])
        return ret.get('ImageSignature', [])

    def is_developer_mounted(self) -> bool:
        """
        Check if developer image mounted

        Raises:
            MuxError("DeviceLocked")
        """
        return len(self.lookup()) > 0

    def _check_error(self, ret: dict):
        if 'Error' in ret:
            raise Exception(ret['Error'])

    def mount(self,
              image_path: str,
              image_signature_path: str):
        """ Mount developer disk image from local files """
        assert os.path.isfile(image_path), image_path
        assert os.path.isfile(image_signature_path), image_signature_path

        with open(image_signature_path, 'rb') as f:
            signature_content = f.read()

        image_size = os.path.getsize(image_path)

        with open(image_path, "rb") as image_reader:
            return self.mount_fileobj(image_reader, image_size, signature_content)

    def mount_fileobj(self,
                      image_reader: typing.IO,
                      image_size: int,
                      signature_content: bytes,
                      image_type: str = "Developer"):

        ret = self.service.plist_request({
            "Command": "ReceiveBytes",
            "ImageSignature": signature_content,
            "ImageSize": image_size,
            "ImageType": image_type,
        })
        self._check_error(ret)
        assert ret['Status'] == 'ReceiveBytesAck'

        # Send data through SSL
        logging.info("Pushing DeveloperDiskImage.dmg")
        chunk_size = 1 << 14

        while True:
            chunk = image_reader.read(chunk_size)
            if not chunk:
                break
            self.service.sock.sendall(chunk)

        ret = self.service.recv_plist()
        self._check_error(ret)

        assert ret['Status'] == 'Complete'
        logging.info("Push complete")

        self.service.send_plist({
            "Command": "MountImage",
            "ImagePath": "/private/var/mobile/Media/PublicStaging/staging.dimag",
            "ImageSignature": signature_content,
            "ImageType": image_type,
        })
        ret = self.service.recv_plist()
        if 'DetailedError' in ret:
            if 'is already mounted at /Developer' in ret['DetailedError']:
                raise Exception("DeveloperImage is already mounted")



import enum


class DgbFuncQual(enum.Enum):
    DBG_FUNC_NONE = 0
    DBG_FUNC_START = 1
    DBG_FUNC_END = 2


class LOG(str, enum.Enum):
    Instrument = 'Instrument'
    LockDown = "LockDown"
    xctest = "xctest"
    USBMux = 'USBMux'
    Mobile = 'Mobile'
    DTXMsg = 'DTXMsg'


class LockdownService(str, enum.Enum):
    MobileLockdown = 'com.apple.mobile.lockdown'
    MobileWirelessLockdown = "com.apple.mobile.wireless_lockdown"

    MobileScreenshotr = "com.apple.mobile.screenshotr"  # 截图服务
    MobileHouseArrest = "com.apple.mobile.house_arrest"  # 访问文件内的沙箱
    AppleAFC = "com.apple.afc"  # 访问系统资源
    InstrumentsRemoteServer = "com.apple.instruments.remoteserver"
    InstrumentsRemoteServerSecure = "com.apple.instruments.remoteserver.DVTSecureSocketProxy"
    TestmanagerdLockdown = "com.apple.testmanagerd.lockdown"
    TestmanagerdLockdownSecure = "com.apple.testmanagerd.lockdown.secure"


class InstrumentsService(str, enum.Enum):
    DeviceInfo = 'com.apple.instruments.server.services.deviceinfo'  # 获取设备信息
    # _selector
    # - machTimeInfo
    # - runningProcesses
    # - nameForGID:
    # - execnameForPid:
    # - isRunningPid:
    # - machKernelName
    # - symbolicatorSignatureForPid: trackingSelector:
    # - unregisterSignatureTrackingForPid:
    # - enableExpiredPidTracking:
    # - symbolicatorSignaturesForExpiredPids
    # - directoryListingForPath:
    # - iconDescriptionFileForAppPath:
    # - hardwareInformation
    # - traceCodesFile
    # - kpepDatabase
    # - sysmonProcessAttributes
    # - sysmonSystemAttributes
    # - sysmonCoalitionAttributes
    # - systemInformation
    # - networkInformation
    # - nameForUID:
    ProcessControl = "com.apple.instruments.server.services.processcontrol"  # 控制应用进程
    # _selector
    # - stopObservingPid:
    # - launchSuspendedProcessWithDevicePath:bundleIdentifier:environment:arguments:options:
    # - sendSignal:toPid:
    # - startObservingPid:
    # - suspendPid:
    # - resumePid:
    # - killPid:
    # - sendProcessControlEvent:toPid:
    Sysmontap = "com.apple.instruments.server.services.sysmontap"  # 获取应用性能数据
    # _selector
    # - setConfig: {'ur,'bm','procAttrs','sysAttrs','cpuUsage','sampleInterval'}
    # - fetchDataNow
    # - pause
    # - stop
    # - start
    # - unpause
    Networking = 'com.apple.instruments.server.services.networking'  # 全局网络数据
    # _selector
    # - replayLastRecordedSession
    # - stopMonitoring
    # - setTargetPID:
    # - startMonitoring
    MobileNotifications = 'com.apple.instruments.server.services.mobilenotifications'  # 监控应用状态
    # _selector
    # - setApplicationStateNotificationsEnabled:
    # - setMemoryNotificationsEnabled:
    GraphicsOpengl = "com.apple.instruments.server.services.graphics.opengl"  # 获取 FPS
    # _selector
    # - startSamplingAtTimeInterval:processIdentifier:
    # - startSamplingAtTimeInterval:
    # - availableStatistics
    # - driverNames
    # - valueForSwitch:
    # - setValue: forSwitchName:
    # - setSamplingRate:
    # - stopSampling
    # - cleanup
    ApplicationListing = "com.apple.instruments.server.services.device.applictionListing"  # 获取应用数据
    # _selector
    # - installedApplicationsMatching: registerUpdateToken:
    # - unregisterUpdateToken:
    XcodeNetworkStatistics = 'com.apple.xcode.debug-gauge-data-providers.NetworkStatistics'  # 获取单进程网络数据
    CoreProfileSessionTap = "com.apple.instruments.server.services.coreprofilesessiontap"  # 获取内核数据
    # _selector
    # - fetchDataNow
    # - pause
    # - stop
    # - start
    # - unpause
    # - setConfig:
    Screenshot = 'com.apple.instruments.server.services.screenshot'  # 获取画面
    # _selector
    # - takeScreenshot
    ConditionInducer = "com.apple.instruments.server.services.ConditionInducer"  # 控制手机，比如网络，手机状态
    # - availableConditionInducers
    # - disableActiveCondition
    # - disableConditionWithIdentifier:
    # - enableConditionWithIdentifier: profileIdentifier:
    XcodeEnergy = 'com.apple.xcode.debug-gauge-data-providers.Energy'



import os
import sys

import click
sys.path.append(os.getcwd())

from ios_device.cli.instruments import cli as instruments_cli
from ios_device.cli.mobile import cli as mobile_cli


def cli():
    cli_commands = click.CommandCollection(sources=[
        instruments_cli,mobile_cli
    ])
    cli_commands()


if __name__ == '__main__':
    cli()
